import { Container } from "inversify";
import { ApiApp } from "./ApiApp";
import { AppConfig } from "./model/AppConfig";
import { ApiRequest } from "./model/ApiRequest";
/**
 * Impementation of the `ApiApp` class that handles native
 * AWS Lambda requests and can be used to provide a Lambda
 * function handler.
 *
 * The `run` method is the function handler entrypoint.
 */
export declare class ApiLambdaApp extends ApiApp {
    /**
     * Create a new lambda app.
     *
     * @param controllersPath (Optional) Paths to the directories that contain controller `js` files.
     *                        Required if the default `Container` is used, or the provided
     *                        `Container` instance has its `autoBindInjectable` flag set to `true`.
     *                        Ignored if the provided `Container` instance has its `autoBindInjectable`
     *                        flag set to `false`.
     * @param appConfig (Optional) Application config to pass to `lambda-api`.
     * @param appContainer (Optional) `InversifyJS` IOC `Container` instance which can build
     *                     controllers and error interceptors.
     */
    constructor(controllersPath?: string[], appConfig?: AppConfig, appContainer?: Container);
    /**
     * Process the passed lambda event and context as a synchronous HTTP request.
     *
     * @param request API Gateway or ALB request.
     * @param context Request context.
     * @returns The response.
     */
    run(event: ApiRequest, context: any): Promise<import("./ts-lambda-api").ApiResponse>;
}
