"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.apiOperation = void 0;
const util_1 = require("util");
const DecoratorRegistry_1 = require("../../reflection/DecoratorRegistry");
/**
 * Decorator that can be placed on an endpoint to describe it in any generated
 * OpenAPI specification.
 *
 * @param apiOperationInfo Information about this api operation; will be merged with
 *                         existing info if present, replacing any existing properties,
 *                         if provided in this parameter.
 */
function apiOperation(apiOperationInfo) {
    return (classDefinition, methodName) => {
        let controller = DecoratorRegistry_1.DecoratorRegistry.getOrCreateController(classDefinition.constructor);
        let endpoint = DecoratorRegistry_1.DecoratorRegistry.getOrCreateEndpoint(controller, methodName);
        if (DecoratorRegistry_1.DecoratorRegistry.getLogger().debugEnabled()) {
            DecoratorRegistry_1.DecoratorRegistry.getLogger().debug("@apiOperation(%s) decorator executed for endpoint: %s", (0, util_1.inspect)(apiOperationInfo), endpoint.name);
        }
        endpoint.getOrCreateApiOperationInfo().mergeInfo(apiOperationInfo);
    };
}
exports.apiOperation = apiOperation;
//# sourceMappingURL=apiOperation.js.map