"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenApiGenerator = void 0;
const oas31_1 = require("openapi3-ts/oas31");
const DecoratorRegistry_1 = require("../reflection/DecoratorRegistry");
const BasicAuthFilter_1 = require("../security/BasicAuthFilter");
const jsonUtils_1 = require("../../util/jsonUtils");
const timed_1 = require("../../util/timed");
class OpenApiGenerator {
    constructor(appConfig, middlewareRegistry, logFactory) {
        this.appConfig = appConfig;
        this.middlewareRegistry = middlewareRegistry;
        this.logger = logFactory.getLogger(OpenApiGenerator);
    }
    buildOpenApiSpec() {
        this.logger.debug("Building raw OpenAPI spec");
        return this.generateApiOpenApiSpecBuilder(this.appConfig, this.middlewareRegistry).getSpec();
    }
    async exportOpenApiSpec(format = "json") {
        let openApiBuilder = this.generateApiOpenApiSpecBuilder(this.appConfig, this.middlewareRegistry);
        if (format === "json") {
            this.logger.debug("Exporting OpenAPI spec as JSON");
            return openApiBuilder.getSpecAsJson();
        }
        try {
            this.logger.debug("Exporting OpenAPI spec as YAML");
            const jsyaml = await Promise.resolve().then(() => __importStar(require("js-yaml")));
            return jsyaml.dump(openApiBuilder.getSpec());
        }
        catch (ex) {
            this.logger.errorWithStack("Error exporting OpenAPI spec as YAML, " +
                "ensure that the 'js-yaml' module is installed", ex);
            throw ex;
        }
    }
    generateApiOpenApiSpecBuilder(appConfig, middlewareRegistry) {
        let openApiBuilder = oas31_1.OpenApiBuilder.create();
        let paths = {};
        let tags = {};
        this.logger.debug("Generating OpenAPI spec");
        if (appConfig.name) {
            this.logger.trace("title: %s", appConfig.name);
            openApiBuilder.addTitle(appConfig.name);
        }
        if (appConfig.version) {
            this.logger.trace("version: %s", appConfig.version);
            openApiBuilder.addVersion(appConfig.version);
        }
        if (appConfig.base) {
            this.logger.trace("base URL: %s", appConfig.base);
            openApiBuilder.addServer({
                url: appConfig.base
            });
        }
        this.discoverSecuritySchemes(openApiBuilder, middlewareRegistry);
        this.discoverTagsAndPaths(paths, tags);
        // add all discovered tags
        for (let tag in tags) {
            if (!tags.hasOwnProperty(tag)) {
                continue;
            }
            this.logger.debug("Adding OpenAPI spec tag: %s", tag);
            openApiBuilder.addTag(tags[tag]);
        }
        // add all discovered paths
        for (let path in paths) {
            if (!paths.hasOwnProperty(path)) {
                continue;
            }
            this.logger.debug("Adding OpenAPI spec path: %s", path);
            openApiBuilder.addPath(path, paths[path]);
        }
        return openApiBuilder;
    }
    discoverSecuritySchemes(openApiBuilder, middlewareRegistry) {
        for (let authFilter of middlewareRegistry.authFilters) {
            let constructor = authFilter.constructor;
            let authFilterInfo = OpenApiGenerator.AUTH_FILTERS[constructor.name];
            if (!authFilterInfo && (authFilter instanceof BasicAuthFilter_1.BasicAuthFilter)) {
                authFilterInfo = OpenApiGenerator.AUTH_FILTERS[BasicAuthFilter_1.BasicAuthFilter.name];
            }
            if (authFilterInfo) {
                this.logger.trace("Adding OpenAPI spec security scheme: %s", authFilterInfo.name);
                openApiBuilder = openApiBuilder.addSecurityScheme(authFilterInfo.name, authFilterInfo.securitySchemeInfo);
            }
        }
    }
    discoverTagsAndPaths(paths, tags) {
        for (let endpoint in OpenApiGenerator.ENDPOINTS) {
            if (!OpenApiGenerator.ENDPOINTS.hasOwnProperty(endpoint)) {
                continue;
            }
            let endpointInfo = OpenApiGenerator.ENDPOINTS[endpoint];
            if (endpointInfo.apiIgnore || (endpointInfo.controller && endpointInfo.controller.apiIgnore)) {
                continue;
            }
            if (endpointInfo.controller) {
                this.addTagIfPresent(tags, endpointInfo.controller);
            }
            this.addEndpoint(paths, endpointInfo);
        }
    }
    addTagIfPresent(tags, controller) {
        if (!controller.apiName || tags[controller.apiName]) {
            // tag name not defined or already recorded
            return;
        }
        this.logger.trace("Adding OpenAPI spec tag for controller: %s", controller.name);
        tags[controller.apiName] = {
            description: controller.apiDescription || "",
            name: controller.apiName
        };
    }
    addEndpoint(paths, endpointInfo) {
        let path = endpointInfo.fullPath;
        if (path.length > 1 && path.endsWith("/")) {
            // trim trailing forward slash from path
            path = path.substring(0, path.length - 1);
        }
        // replace :param with {param} to match OpenAPI spec
        path = path.replace(/:([^\/]*)(\/?)/g, "{$1}$2");
        let pathInfo = paths[path] || {};
        let endpointMethod = endpointInfo.httpMethod.toLowerCase();
        let endpointOperation = {
            responses: {}
        };
        this.logger.trace("Adding path for endpoint: %s", endpointInfo.name);
        if (endpointMethod !== "get" && endpointMethod !== "delete") {
            this.setRequestInfo(endpointOperation, endpointInfo);
        }
        if (endpointInfo.responseContentType) {
            this.logger.trace("Setting path response content type: %s", endpointInfo.responseContentType);
            // user declared response content type
            this.setEndpointResponseContentType(endpointOperation, endpointInfo.responseContentType);
        }
        else {
            this.logger.trace("Setting path response content type to default: application/json");
            // default response content type
            this.setEndpointResponseContentType(endpointOperation, "application/json" // use lambda-api content-type default
            );
        }
        if (endpointInfo.apiOperationInfo) {
            this.logger.trace("Setting user defined endpoint info for endpoint: %s", endpointInfo.name);
            // user declared endpoint info
            this.addEndpointOperationInfo(endpointInfo, endpointOperation);
        }
        this.addParametersToEndpoint(endpointOperation, endpointInfo);
        if (endpointInfo.getControllerPropOrDefault(c => c.apiName)) {
            // associate endpoint with controller tag name
            endpointOperation.tags = [endpointInfo.controller.apiName];
        }
        else if (endpointInfo.controller) {
            endpointOperation.tags = [endpointInfo.controller.name.replace(/Controller$/, "")];
        }
        else {
            endpointOperation.tags = [endpointInfo.name];
        }
        if (endpointInfo.noAuth) {
            this.logger.trace("Excluding endpoint from security because of noAuth flag: %s", endpointInfo.name);
            // no authentication required, exclude from global security constraints
            endpointOperation.security = [];
        }
        pathInfo[endpointMethod] = endpointOperation;
        paths[path] = pathInfo;
    }
    setRequestInfo(endpointOperation, endpointInfo) {
        let requestInfo = endpointInfo.apiRequestInfo;
        let operationRequestType;
        this.logger.trace("Setting request info for endpoint: %s", endpointInfo.name);
        if (requestInfo) {
            operationRequestType = requestInfo.contentType ||
                endpointInfo.requestContentType ||
                "application/json";
        }
        operationRequestType = operationRequestType ||
            endpointInfo.requestContentType;
        if (!operationRequestType) {
            return;
        }
        operationRequestType = operationRequestType.toLowerCase();
        // user declared request content type
        let mediaTypeObject = this.setEndpointRequestContentType(endpointOperation, operationRequestType, requestInfo);
        requestInfo.contentType = operationRequestType;
        if (!requestInfo || (!requestInfo.type && !requestInfo.class)) {
            this.logger.trace("No request type info found for endpoint: %s", endpointInfo.name);
            // no type info for request body
            return;
        }
        this.logger.trace("Adding request schema for endpoint: %s", endpointInfo.name);
        mediaTypeObject.schema = {};
        if (requestInfo.type) {
            mediaTypeObject.schema = this.getPrimitiveTypeSchema(requestInfo);
        }
        else if (requestInfo.class) {
            this.addClassToMediaTypeObject(mediaTypeObject, requestInfo, operationRequestType);
        }
        if (requestInfo.example) {
            this.logger.trace("Using user defined request example for endpoint: %s", endpointInfo.name);
            mediaTypeObject.example = requestInfo.example;
        }
        else if (mediaTypeObject.schema) {
            let schema = mediaTypeObject.schema;
            if (schema && schema.example) {
                mediaTypeObject.example = schema.example;
            }
        }
    }
    setEndpointRequestContentType(endpointOperation, requestContentType, requestInfo) {
        let requestBody = {
            content: {},
            description: requestInfo ? (requestInfo.description || "") : ""
        };
        let mediaTypeObject = {};
        requestBody.content[requestContentType] = mediaTypeObject;
        endpointOperation.requestBody = requestBody;
        this.logger.trace("Endpoint request content type: %s", requestContentType);
        return mediaTypeObject;
    }
    addEndpointOperationInfo(endpointInfo, endpointOperation) {
        let operationInfo = endpointInfo.apiOperationInfo;
        let responseContentType = endpointInfo.responseContentType || "application/json";
        endpointOperation.summary = operationInfo.name || "";
        endpointOperation.description = operationInfo.description || "";
        if (operationInfo.request) {
            this.setRequestInfo(endpointOperation, endpointInfo);
        }
        for (let statusCode in operationInfo.responses) {
            if (!operationInfo.responses.hasOwnProperty(statusCode)) {
                continue;
            }
            this.setEndpointResponseContentType(endpointOperation, responseContentType, statusCode, operationInfo.responses[statusCode]);
        }
    }
    setEndpointResponseContentType(endpointOperation, responseContentType, statusCode, apiBodyInfo) {
        let response;
        if (apiBodyInfo) {
            this.logger.trace("Adding user defined response info");
            // user defined response body info
            let responseContent = {};
            let mediaTypeObject = {};
            apiBodyInfo.contentType = (apiBodyInfo.contentType || responseContentType).toLowerCase();
            let responseType = apiBodyInfo.contentType;
            responseContent[responseType] = mediaTypeObject;
            response = {
                content: responseContent,
                description: apiBodyInfo.description || ""
            };
            if (apiBodyInfo.type) {
                this.logger.trace("Building response schema from primitive type");
                mediaTypeObject.schema = this.getPrimitiveTypeSchema(apiBodyInfo);
            }
            else if (apiBodyInfo.class) {
                this.logger.trace("Building response schema from class type");
                this.addClassToMediaTypeObject(mediaTypeObject, apiBodyInfo, responseType);
            }
            if (apiBodyInfo.example) {
                this.logger.trace("Using user defined response example: %s", apiBodyInfo.example);
                mediaTypeObject.example = apiBodyInfo.example;
            }
            else if (mediaTypeObject.schema) {
                let schema = mediaTypeObject.schema;
                if (schema && schema.example) {
                    this.logger.trace("Using schema provided response example: %s", schema.example);
                    mediaTypeObject.example = schema.example;
                }
            }
            else {
                this.logger.trace("No example defined for OpenAPI response request");
            }
        }
        else {
            this.logger.trace("Setting response content type: %s", responseContentType);
            // response content type only
            let responseContent = {};
            responseContent[responseContentType] = {};
            response = {
                content: responseContent,
                description: "" // required or swagger ui will throw errors
            };
        }
        if (statusCode) {
            this.logger.trace("Setting response for HTTP %s: %j", statusCode, response);
            endpointOperation.responses[statusCode] = response;
        }
        else {
            this.logger.trace("Setting path default response: %j", response);
            endpointOperation.responses.default = response;
        }
    }
    getPrimitiveTypeSchema(apiBodyInfo) {
        let type = apiBodyInfo.type.toLowerCase();
        if (!OpenApiGenerator.OPEN_API_TYPES.includes(type)) {
            this.logger.trace("Skipping adding unknown body type to OpenAPI spec: %s", type);
            return;
        }
        let schemaType = OpenApiGenerator.OPEN_API_SCHEMA_TYPE_MAP[type];
        if (type !== "file") {
            this.logger.trace("Setting body to type: %s", type);
            let schema = {
                type: schemaType
            };
            if (schemaType === "array") {
                this.addPrimitiveArrayInfoToSchema(schema, apiBodyInfo.type);
            }
            if (schemaType === "array" || schemaType === "object") {
                if (!apiBodyInfo.contentType ||
                    apiBodyInfo.contentType.toLowerCase() !== "application/json") {
                    // exclude object or array examples if content type is non-json
                    return schema;
                }
            }
            schema.example = this.getPrimitiveTypeExample(type);
            return schema;
        }
        else {
            this.logger.trace("Setting body to file");
            return {
                format: "binary",
                type: schemaType
            };
        }
    }
    getPrimitiveTypeExample(type) {
        let example = OpenApiGenerator.OPEN_API_TYPE_EXAMPLES[type];
        this.logger.trace("Adding body example: %s", example);
        return (0, jsonUtils_1.toJson)(example);
    }
    addPrimitiveArrayInfoToSchema(schema, type) {
        let itemType = type.replace("-array", "");
        let itemSchemaType = OpenApiGenerator.OPEN_API_SCHEMA_TYPE_MAP[itemType];
        // ensure array items have a type
        schema.items = {
            type: itemSchemaType
        };
        if (itemSchemaType === "object") {
            // ensure object items can have any properties
            schema.items.additionalProperties = true;
        }
        else if (itemSchemaType === "array") {
            // ensure arrays inside an array have any item types
            schema.items.items = {
                additionalProperties: true,
                type: "object"
            };
        }
    }
    addClassToMediaTypeObject(mediaTypeObject, apiBodyInfo, responseContentType) {
        let clazz = apiBodyInfo.class;
        let instance;
        if ((typeof clazz.example) === "function") {
            this.logger.trace("Getting body info from 'example' method of class: %s", apiBodyInfo.class.name);
            instance = clazz.example();
        }
        else {
            this.logger.trace("Getting body info from default contructor of class: %s", apiBodyInfo.class.name);
            instance = new clazz();
        }
        let instanceType = "object";
        if (!(instance instanceof clazz)) {
            instanceType = this.getInstanceType(instance);
            if (!OpenApiGenerator.OPEN_API_TYPES.includes(instanceType)) {
                // not a supported type or null/undefined, ommit this property
                this.logger.trace("Ignoring class '%s' as it is of an invalid type: %s", apiBodyInfo.class.name, instanceType);
                return;
            }
        }
        let schema = {
            type: instanceType
        };
        mediaTypeObject.schema = schema;
        if (responseContentType === "application/json") {
            let exampleJson = (0, jsonUtils_1.toJson)(instance);
            this.logger.trace("Adding JSON body example: %s", exampleJson);
            mediaTypeObject.example = exampleJson;
            schema.example = exampleJson;
        }
        if (instanceType === "object") {
            this.addObjectPropertiesToSchema(schema, instance);
        }
        else if (instanceType === "array") {
            if (instance.length > 0) {
                this.addArrayToSchema(schema, instance);
            }
            else {
                this.logger.trace("Ignoring class '%s' array as it has no items in it's example", apiBodyInfo.class.name);
                delete mediaTypeObject.schema;
                return;
            }
        }
        else {
            schema.example = instance;
            mediaTypeObject.example = instance;
        }
    }
    addObjectPropertiesToSchema(schema, instance) {
        let objectProperties = Object.getOwnPropertyNames(instance);
        for (let property of objectProperties) {
            let type = this.getTypeOfInstanceProperty(instance, property);
            if (!OpenApiGenerator.OPEN_API_TYPES.includes(type)) {
                // not a supported type or null/undefined, ommit this property
                this.logger.trace("Ignoring property '%s' as it is of an invalid type: %s", property, type);
                continue;
            }
            let propertySchema = {
                type
            };
            if (type === "object") {
                // get schema for property object
                propertySchema.example = (0, jsonUtils_1.toJson)(instance[property]);
                this.addObjectPropertiesToSchema(propertySchema, instance[property]);
            }
            else if (type === "array") {
                if (instance[property].length > 0) {
                    if (!this.addArrayToSchema(propertySchema, instance[property])) {
                        // unsupported array item type or null/undefined, ommit this property
                        continue;
                    }
                }
                else {
                    // no way to determine array item type, so better to ommit this property
                    continue;
                }
            }
            else {
                propertySchema.example = instance[property];
            }
            if (!schema.properties) {
                schema.properties = {};
            }
            schema.properties[property] = propertySchema;
        }
    }
    addArrayToSchema(propertySchema, instance) {
        let itemType = this.getInstanceType(instance[0]);
        if (!OpenApiGenerator.OPEN_API_TYPES.includes(itemType)) {
            return false;
        }
        propertySchema.example = (0, jsonUtils_1.toJson)(instance);
        propertySchema.items = {
            type: itemType
        };
        if (itemType === "object") {
            propertySchema.items.example = (0, jsonUtils_1.toJson)(instance[0]);
            // get schema for array item type
            this.addObjectPropertiesToSchema(propertySchema.items, instance[0]);
        }
        else if (itemType === "array") {
            if (instance.length > 0) {
                this.addArrayToSchema(propertySchema.items, instance[0]);
            }
        }
        else {
            propertySchema.items.example = instance[0];
        }
        return true;
    }
    getTypeOfInstanceProperty(instance, property) {
        return this.getInstanceType(instance[property]);
    }
    getInstanceType(instance) {
        if (instance === null || typeof instance === 'undefined') {
            return null;
        }
        let type = ((typeof instance)).toLowerCase();
        if (type === "object" && Array.isArray(instance)) {
            // the property is actually an array
            type = "array";
        }
        return type;
    }
    addParametersToEndpoint(endpointOperation, endpointInfo) {
        endpointOperation.parameters = [];
        endpointInfo.parameterExtractors.forEach(p => {
            if (p.source === "virtual") {
                return;
            }
            if (p.source === "header" &&
                OpenApiGenerator.FORBIDDEN_HEADER_PARAMS.includes(p.name.toLowerCase())) {
                this.logger.debug("Parameter for header %s is forbidden by the OpenAPI v3 Spec", p.name);
                return;
            }
            let paramInfo = {
                in: p.source,
                name: p.name,
                schema: {} // required, or breaks query parameters
            };
            if (p.apiParamInfo) {
                this.addEndpointParameterInfo(paramInfo, p.apiParamInfo);
            }
            if (p.source === "path") {
                // swagger ui will break if this is not set to true
                paramInfo.required = true;
            }
            endpointOperation.parameters.push(paramInfo);
        });
        this.logger.trace("Setting path parameters: %j", endpointOperation.parameters);
    }
    addEndpointParameterInfo(paramInfo, apiParamInfo) {
        // remove any previously written schema
        delete paramInfo.schema;
        let paramContentType = apiParamInfo.contentType;
        let mediaTypeObject = {};
        if (paramContentType) {
            // we have a content type, use it in the param info
            paramInfo.content = {};
            paramInfo.content[paramContentType] = mediaTypeObject;
        }
        if (apiParamInfo.type) {
            mediaTypeObject.schema = this.getPrimitiveTypeSchema(apiParamInfo);
            if (apiParamInfo.type.endsWith("array")) {
                this.setParamValueStyle(paramInfo, apiParamInfo);
            }
        }
        else if (apiParamInfo.class) {
            this.addClassToMediaTypeObject(mediaTypeObject, apiParamInfo, paramContentType);
            this.setParamValueStyle(paramInfo, apiParamInfo);
        }
        else {
            apiParamInfo.type = "string";
            mediaTypeObject.schema = this.getPrimitiveTypeSchema(apiParamInfo);
        }
        if (apiParamInfo.example) {
            mediaTypeObject.example = apiParamInfo.example;
        }
        if (!paramInfo.content) {
            // we don't have info in content, place schema and example inside param info
            if (mediaTypeObject.schema) {
                paramInfo.schema = mediaTypeObject.schema;
            }
            if (mediaTypeObject.example) {
                paramInfo.example = mediaTypeObject.example;
            }
        }
        if (typeof apiParamInfo.required === "boolean") {
            paramInfo.required = apiParamInfo.required;
        }
        if (apiParamInfo.description) {
            paramInfo.description = apiParamInfo.description;
        }
    }
    setParamValueStyle(paramInfo, apiParamInfo) {
        if (paramInfo.content) {
            // can't set style or explode if a content type is provided
            return;
        }
        if (typeof apiParamInfo.explode === "boolean") {
            paramInfo.explode = apiParamInfo.explode;
        }
        if (!apiParamInfo.style) {
            return;
        }
        if (paramInfo.in === "header" && apiParamInfo.style !== "simple") {
            this.logger.debug("Header parameters only support the simple style, style '%s' ignored", apiParamInfo.style);
            return;
        }
        if (paramInfo.in === "path") {
            if (!OpenApiGenerator.VALID_PATH_PARAM_STYLES.includes(apiParamInfo.style)) {
                this.logger.debug("'%s' is not a valid path param style", apiParamInfo.style);
                return;
            }
        }
        if (paramInfo.in === "query") {
            if (!apiParamInfo.class && apiParamInfo.style === "deepObject") {
                this.logger.debug("'deepObject' style is only supported for object query string parameters");
                return;
            }
            if (apiParamInfo.class &&
                !OpenApiGenerator.VALID_QUERY_OBJECT_PARAM_STYLES.includes(apiParamInfo.style)) {
                this.logger.debug("'%s' is not a valid style for object query string parameters", paramInfo.style);
                return;
            }
        }
        paramInfo.style = apiParamInfo.style;
    }
}
exports.OpenApiGenerator = OpenApiGenerator;
OpenApiGenerator.ENDPOINTS = DecoratorRegistry_1.DecoratorRegistry.Endpoints;
OpenApiGenerator.AUTH_FILTERS = DecoratorRegistry_1.DecoratorRegistry.AuthFilters;
OpenApiGenerator.OPEN_API_TYPES = [
    "array",
    "array-array",
    "boolean",
    "boolean-array",
    "double",
    "double-array",
    "file",
    "int",
    "int-array",
    "number",
    "number-array",
    "object",
    "object-array",
    "string",
    "string-array"
];
OpenApiGenerator.OPEN_API_SCHEMA_TYPE_MAP = {
    "array": "array",
    "array-array": "array",
    "boolean": "boolean",
    "boolean-array": "array",
    "double": "number",
    "double-array": "array",
    "file": "string",
    "int": "number",
    "int-array": "array",
    "number": "number",
    "number-array": "array",
    "object": "object",
    "object-array": "array",
    "string": "string",
    "string-array": "array"
};
OpenApiGenerator.OPEN_API_TYPE_EXAMPLES = {
    "array": [],
    "array-array": [[], [], []],
    "boolean": true,
    "boolean-array": [true, false, true],
    "double": 1.1,
    "double-array": [1.1, 2.2, 3.3],
    "file": "upload a file",
    "int": 1,
    "int-array": [1, 2, 3],
    "number": 1.1,
    "object": {},
    "object-array": [{}, {}, {}],
    "string": "a string",
    "string-array": ["1st string", "2nd string", "3rd string"]
};
OpenApiGenerator.FORBIDDEN_HEADER_PARAMS = [
    "accept",
    "content-type",
    "authorization"
];
OpenApiGenerator.VALID_PATH_PARAM_STYLES = [
    "simple",
    "label",
    "matrix"
];
OpenApiGenerator.VALID_QUERY_OBJECT_PARAM_STYLES = [
    "form",
    "deepObject"
];
__decorate([
    timed_1.timed,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], OpenApiGenerator.prototype, "buildOpenApiSpec", null);
__decorate([
    timed_1.timed,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], OpenApiGenerator.prototype, "exportOpenApiSpec", null);
//# sourceMappingURL=OpenApiGenerator.js.map